@@ -3,29 +3,27 @@ package Mojolicious::Plugin::Qooxdoo::JsonRpcController;
 use strict;
 use warnings;
 
-use Mojo::JSON qw(encode_json decode_json);
+use Mojo::JSON;
 use Mojo::Base 'Mojolicious::Controller';
 use Encode;
 
 
-has toUTF8 => sub { find_encoding('utf8') };
+our $toUTF8 = find_encoding('utf8');
 
-our $VERSION = '0.10';
+our $VERSION = '0.2';
+
+has 'JSON' => sub { Mojo::JSON->new };
 
 has 'service';
 
-has 'crossDomain';
-has 'requestId';
-has 'methodName';
-
-has log => sub { shift->app->log };
-
 sub dispatch {
     my $self = shift;
     
     # We have to differentiate between POST and GET requests, because
     # the data is not sent in the same place..
-    my $log = $self->log;
+    my $log = $self->app->log;
+
+    my $json = $self->JSON;
 
     # send warnings to log file preserving the origin
     local $SIG{__WARN__} = sub {
@@ -34,43 +32,44 @@ sub dispatch {
         @_ = ($log, $message);
         goto &Mojo::Log::warn;
     };
+    my $id;    
     my $data;
+    my $cross_domain;
     for ( $self->req->method ){
         /^POST$/ && do {
             # Data comes as JSON object, so fetch a reference to it
-            $data = eval { decode_json($self->req->body) };
-	    	if ($@) {
-				my $error = "Invalid json string: " . $@;
-				$log->error($error);
-				$self->render(text => $error, status=>500);
-				return;
-	    	};
-            $self->requestId($data->{id});
-            $self->crossDomain(0);
-            last;
+            $data           = $json->decode($self->req->body) or 
+	    do {
+		my $error = "Invalid json string: " . $json->error;
+		$log->error($error);
+		$self->render(text => $error, status=>500);
+		return;
+	    };
+            $id             = $data->{id};
+            $cross_domain   = 0;
+            next;
         };
         /^GET$/ && do {
+            $data= $json->decode(
+                $self->param('_ScriptTransport_data')
+            ) or
+	    do {
+		my $error = "Invalid json string: " . $json->error;
+		$log->error($error);
+		$self->render(text => $error, status=>500);
+		return;
+	    };
 
-            $data= eval { decode_json($self->param('_ScriptTransport_data')) };
-
-            if ($@) {
-				my $error = "Invalid json string: " . $@;
-				$log->error($error);
-				$self->render(text => $error, status=>500);
-				return;
-	    	};
-
-            $self->requestId($self->param('_ScriptTransport_id')) ;
-            $self->crossDomain(1);
-            last;
+            $id = $self->param('_ScriptTransport_id') ;
+            $cross_domain   = 1;
+            next;
         };
-
         my $error = "request must be POST or GET. Can't handle '".$self->req->method."'";
         $log->error($error);
         $self->render(text => $error, status=>500);
         return;
     }        
-    if (not defined $self->requestId){
+    if (not defined $id){
         my $error = "Missing 'id' property in JsonRPC request.";
         $log->error($error);
         $self->render(text => $error, status=>500);
@@ -93,12 +92,11 @@ sub dispatch {
         $self->render(text => $error, status=>500);
         return;
     };
-    $self->methodName($method);
-
+    
     my $params  = $data->{params} || []; # is a reference, so "unpack" it
  
     # invocation of method in class according to request 
-    my $reply = eval {
+    my $reply = eval{
         # make sure there are not foreign signal handlers
         # messing with our problems
         local $SIG{__DIE__};
@@ -130,75 +128,56 @@ sub dispatch {
              code=> 4
         } if not $self->can($method);
 
-        $log->debug("call $method(".encode_json($params).")");
+        $log->debug("call $method(".$json->encode($params).")");
         # reply
         no strict 'refs';
         $self->$method(@$params);
     };
-    if ($@){
-        $self->renderJsonRpcError($@);
-    }
-    else {
-    	# do NOT render if
-    	if (not $self->stash->{'mojo.rendered'}){
-    		$self->renderJsonRpcResult($reply);
-    	}
-    }
-}
-
-sub renderJsonRpcResult {
-	my $self = shift;
-	my $data = shift;
-    my $reply = encode_json({ id => $self->requestId, result => $data });
-    $self->log->debug("return ".$reply);
-    $self->finalizeJsonRpcReply($reply);
-}
-
-sub renderJsonRpcError {
-	my $self = shift;
-	my $exception = shift;
-	my $error;
-	for (ref $exception){
-        /HASH/ && $exception->{message} && do {
-            $error = {
-                origin => $exception->{origin} || 2, 
-                message => $exception->{message}, 
-                code=>$exception->{code}
+       
+    if ($@){ 
+        my $error;
+        for (ref $@){
+            /HASH/ && $@->{message} && do {
+                $error = {
+                     origin => $@->{origin} || 2, 
+                     message => $@->{message}, 
+                     code=>$@->{code}
+                };
+                last;
+            };
+            /.+/ && $@->can('message') && $@->can('code') && do {
+                $error = {
+                      origin => 2, 
+                      message => $@->message(), 
+                      code=>$@->code()
+                };
+                last;
             };
-            last;
-        };
-        /.+/ && $exception->can('message') && $exception->can('code') && do {
             $error = {
                 origin => 2, 
-                message => $exception->message(), 
-                code=>$exception->code()
+                message => "error while processing ${service}::$method: $@", 
+                code=> 9999
             };
-            last;
-        };
-        $error = {
-            origin => 2, 
-            message => "error while processing ".$self->service."::".$self->methodName.": $exception", 
-            code=> 9999
-        };
+        }
+        $reply = $json->encode({ id => $id, error => $error });
+        $log->error("JsonRPC Error $error->{code}: $error->{message}");
+    }
+    else {
+        $reply = $json->encode({ id => $id, result => $reply });
+        $log->debug("return ".$reply);
     }
-    $self->log->error("JsonRPC Error $error->{code}: $error->{message}");
-    $self->finalizeJsonRpcReply(encode_json({ id => $self->requestId, error => $error}));
-}
 
-sub finalizeJsonRpcReply {
-	my $self  = shift;
-	my $reply = shift;
-    if ($self->crossDomain){
+    if ($cross_domain){
         # for GET requests, qooxdoo expects us to send a javascript method
         # and to wrap our json a litte bit more
         $self->res->headers->content_type('application/javascript; charset=utf-8');
-        $reply = "qx.io.remote.transport.Script._requestFinished( ".$self->requestId.", " . $reply . ");";
+        $reply = "qx.io.remote.transport.Script._requestFinished( $id, " . $reply . ");";
     } else {
         $self->res->headers->content_type('application/json; charset=utf-8');
     }    
     # the render takes care of encoding the output, so make sure we re-decode
     # the json stuf
-    $self->render(text => $self->toUTF8->decode($reply));
+    $self->render(text => $toUTF8->decode($reply));
 }
 
 1;
@@ -206,7 +185,7 @@ sub finalizeJsonRpcReply {
 
 =head1 NAME
 
-Mojolicious::Plugin::Qooxdoo::JsonRpcController - A controller base class for Qooxdoo JSON-RPC Calls
+Mojolicious::Plugin::Qooxdoo::JsonRpcController - A controller base class for Qooxdoo Json Rpc Calls
 
 =head1 SYNOPSIS
 
@@ -231,12 +210,10 @@ Mojolicious::Plugin::Qooxdoo::JsonRpcController - A controller base class for Qo
  
  has service => sub { 'Test' };
  
- out %allow = ( echo => 1, bad =>  1, async => 1);
-
  sub allow_rpc_access {
     my $self = shift;
     my $method = shift;
-    return $allow{$method};;
+    return $method eq 'echo';
  }
 
  sub echo {
@@ -252,19 +229,6 @@ Mojolicious::Plugin::Qooxdoo::JsonRpcController - A controller base class for Qo
     die { code => 1234, message => 'another way to die' };
  }
 
- sub async {
-    my $self=shift;
-    $self->render_later;
-    xyzWithCallback(callback=>sub{
-        eval {
-            local $SIG{__DIE__};
-            $self->renderJsonRpcResult('Late Reply');
-        }
-        if ($@) {
-            $self->renderJsonRpcError($@);
-        }
-    });
- }
 
  package MyException;
 
@@ -277,15 +241,15 @@ Mojolicious::Plugin::Qooxdoo::JsonRpcController - A controller base class for Qo
 
 All you have todo to process incoming JSON-RPC requests from a qooxdoo
 application, is to make your controller a child of
-L<Mojolicious::Plugin::Qooxdoo::JsonRpcController>.  And then route all
+L<Mojolicious::Plugin::Qooxdoo::JsonRpcService>.  And then route all
 incoming requests to the inherited dispatch method in the new controller.
 
 If you want your Mojolicious app to also serve the qooxdoo application
-files, you can use L<Mojolicous::Plugin::Qooxdoo> to have everything setup for you.
+files, you can use L<Mojolicous::Plugin::Qooxdoo> to have everyting setup for you.
 
 =head2 Exception processing
 
-Errors within the methods of your controller are handled by an eval call,
+Errors within the methods of your RpcService are handled by an eval call,
 encapsulating the method call.  So if you run into trouble, just C<die>.  If
 if you die with a object providing a C<code> and C<message> property or with
 a hash containing a C<code> and C<message> key, this information will be
@@ -294,20 +258,12 @@ used to populate the JSON-RPC error object returned to the caller.
 =head2 Security
 
 The C<dispatcher> method provided by
-L<Mojolicious::Plugin::Qooxoo::JsonRpcController> calls the C<allow_rpc_access>
+L<Mojolicious::Plugin::Qooxoo::JsonRpcService> calls the C<allow_rpc_access>
 method to check if rpc access should be allowed.  The result of this request
 is NOT cached, so you can use this method to provide dynamic access control
 or even do initialization tasks that are required before handling each
 request.
 
-=head2 Async Processing
-
-If you want to do asyncronoous data processing, call the C<render_later> method
-to let the dispatcher know that it should not bother with trying to render anyting.
-In the callback, call the C<renderJsonRpcResult> method to render your result. Note
-that you have to take care of any exceptions in the callback yourself and use
-the C<renderJsonRpcError> method to send the exception to the client.
-
 =head1 AUTHOR
 
 S<Matthias Bloch, E<lt>matthias@puffin.chE<gt>>,
